"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xmr_crypto_utils_1 = require("@xmr-core/xmr-crypto-utils");
const { derive_key_image_from_tx } = xmr_crypto_utils_1.key_image;
const keyImagesByWalletId = {};
/**
 * @description Performs a memoized computation of a key image
 * @param {KeyImageCache} keyImageCache
 * @param {string} txPubKey
 * @param {number} outIndex
 * @param {string} address
 * @param {string} privViewKey
 * @param {string} pubSpendKey
 * @param {string} privSpendKey
 * @returns
 */
async function genKeyImageFromTx(keyImageCache, txPubKey, outIndex, address, privViewKey, pubSpendKey, privSpendKey, hwdev) {
    const cacheIndex = `${txPubKey}:${address}:${outIndex}`;
    const cachedKeyImage = keyImageCache[cacheIndex];
    if (cachedKeyImage) {
        return cachedKeyImage;
    }
    const { key_image } = await derive_key_image_from_tx(txPubKey, privViewKey, pubSpendKey, privSpendKey, outIndex, hwdev);
    // cache the computed key image
    keyImageCache[cacheIndex] = key_image;
    return key_image;
}
exports.genKeyImageFromTx = genKeyImageFromTx;
/**
 *
 * @description Get a key image cache, that's mapped by address
 * @export
 * @param {string} address
 */
function getKeyImageCache(address) {
    const cacheId = parseAddress(address);
    let cache = keyImagesByWalletId[cacheId];
    if (!cache) {
        cache = {};
        keyImagesByWalletId[cacheId] = cache;
    }
    return cache;
}
exports.getKeyImageCache = getKeyImageCache;
/**
 * @description Clears a key image cache that's mapped by the users address
 *
 *
 * IMPORTANT: Ensure you call this method when you want to clear your wallet from
 * memory or delete it, or else you could leak key images and public addresses.
 * @export
 * @param {string} address
 */
function clearKeyImageCache(address) {
    const cacheId = parseAddress(address);
    delete keyImagesByWalletId[cacheId];
    const cache = keyImagesByWalletId[cacheId];
    if (cache) {
        throw Error("Key image cache still exists after deletion");
    }
}
exports.clearKeyImageCache = clearKeyImageCache;
/**
 * @description Normalize an address before using it to access the key image cache map as a key
 * @param {string} address
 */
function parseAddress(address) {
    // NOTE: making the assumption that public_address is unique enough to identify a wallet for caching....
    // FIXME: with subaddresses, is that still the case? would we need to split them up by subaddr anyway?
    if (!address) {
        throw Error("Address does not exist");
    }
    return address.toString();
}
//# sourceMappingURL=memoized-keyimage-generation.js.map