"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const key_utils_1 = require("../key_utils");
const errors_1 = require("../errors");
const logger_1 = require("../logger");
const arr_utils_1 = require("../arr_utils");
const fee_utils_1 = require("../fee_utils");
const biginteger_1 = require("@xmr-core/biginteger");
const xmr_transaction_1 = require("@xmr-core/xmr-transaction");
const xmr_fast_hash_1 = require("@xmr-core/xmr-fast-hash");
const xmr_crypto_utils_1 = require("@xmr-core/xmr-crypto-utils");
const xmr_constants_1 = require("@xmr-core/xmr-constants");
const xmr_money_1 = require("@xmr-core/xmr-money");
const xmr_crypto_utils_2 = require("@xmr-core/xmr-crypto-utils");
const xmr_str_utils_1 = require("@xmr-core/xmr-str-utils");
// #region totalAmtAndEstFee
/**
 * @description
 * Recalculates the fee and total amount needed for the transaction to be sent. RCT + non sweeping transactions will have their
 * network fee increased if fee calculation based on the number of outputs needed is higher than the passed-in fee. RCT+ sweeping transactions
 * are just checked if they have enough balance to proceed with the transaction. Non-RCT transactions will have no fee recalculation done on them.

 * @export
 * @param {TotalAmtAndEstFeeParams} params
 * @returns
 */
function totalAmtAndEstFee(params) {
    const { baseTotalAmount, networkFee, isRingCT, usingOuts, } = params;
    if (!isRingCT) {
        return { newFee: networkFee, totalAmount: baseTotalAmount };
    }
    /* if (usingOuts.length > 1 && isRingCT )*/
    const { newFee, totalAmount } = estRctFeeAndAmt(params);
    logger_1.Log.Fee.basedOnInputs(newFee, usingOuts);
    return { newFee, totalAmount };
}
exports.totalAmtAndEstFee = totalAmtAndEstFee;
function estRctFeeAndAmt(params) {
    const { mixin, usingOuts, usingOutsAmount, simplePriority, feePerKB, // obtained from server, so passed in
    networkFee, isSweeping, } = params;
    let feeBasedOnOuts = fee_utils_1.calculateFee(feePerKB, xmr_transaction_1.estimateRctSize(usingOuts.length, mixin, 2), fee_utils_1.multiplyFeePriority(simplePriority));
    // if newNeededFee < neededFee, use neededFee instead
    //(should only happen on the 2nd or later times through(due to estimated fee being too low))
    if (feeBasedOnOuts.compare(networkFee) < 0) {
        feeBasedOnOuts = networkFee;
    }
    const [totalAmount, newFee] = isSweeping
        ? estRctSwpingAmt(usingOutsAmount, feeBasedOnOuts)
        : estRctNonSwpAmt(params, feeBasedOnOuts);
    return { totalAmount, newFee };
}
function estRctSwpingAmt(usingOutsAmount, fee) {
    /*
    // When/if sending to multiple destinations supported, uncomment and port this:
    if (dsts.length !== 1) {
        deferred.reject("Sweeping to multiple accounts is not allowed");
        return;
    }
    */
    // feeless total is equivalent to all outputs (since its a sweeping tx)
    // subtracted from the newNeededFee  (either from min tx cost or calculated cost based on outputs)
    const _feelessTotal = usingOutsAmount.subtract(fee);
    // if the feeless total is less than 0 (so sum of all outputs is still less than network fee)
    // then reject tx
    if (_feelessTotal.compare(0) < 1) {
        throw errors_1.ERR.BAL.insuff(usingOutsAmount, fee);
    }
    // otherwise make the total amount the feeless total + the new fee
    const totalAmount = _feelessTotal.add(fee);
    return [totalAmount, fee];
}
function estRctNonSwpAmt(params, fee) {
    const { mixin, remainingUnusedOuts, usingOuts, usingOutsAmount, simplePriority, feelessTotal, feePerKB, } = params;
    // make the current total amount equivalent to the feeless total and the new needed fee
    let currTotalAmount = feelessTotal.add(fee);
    // add outputs 1 at a time till we either have them all or can meet the fee
    // this case can happen when the fee calculated via outs size
    // is greater than the minimum tx fee size,
    // requiring a higher fee, so more outputs (if available)
    // need to be selected to fufill the difference
    let newFee = fee;
    while (usingOutsAmount.compare(currTotalAmount) < 0 &&
        remainingUnusedOuts.length > 0) {
        const out = arr_utils_1.popRandElement(remainingUnusedOuts);
        logger_1.Log.Output.display(out);
        // and recalculate invalidated values
        newFee = fee_utils_1.calculateFee(feePerKB, xmr_transaction_1.estimateRctSize(usingOuts.length, mixin, 2), fee_utils_1.multiplyFeePriority(simplePriority));
        currTotalAmount = feelessTotal.add(newFee);
    }
    const totalAmount = currTotalAmount;
    return [totalAmount, newFee];
}
// #endregion totalAmtAndEstFee
// #region validateAndConstructFundTargets
/**
 * @description
 * 1. Validates the total amount needed for the tx against the available amounts via the sum of all outputs
 * to see if the sender has sufficient funds.
 *
 * 2. Then, a list of sending targets will be constructed, always consisting of the target address and amount they want to send to, and possibly a change address,
 * if the sum of outs is greater than the amount sent + fee needed, and possibly a fake address + 0 amount to keep output uniformity if no change address
 * was generated.
 *
 * @export
 * @param {ConstructFundTargetsParams} params
 * @returns
 */
function validateAndConstructFundTargets(params) {
    const { senderAddress, targetAddress, feelessTotal, totalAmount, usingOutsAmount, isSweeping, isRingCT, nettype, } = params;
    // Now we can validate available balance with usingOutsAmount (TODO? maybe this check can be done before selecting outputs?)
    const outsCmpToTotalAmounts = usingOutsAmount.compare(totalAmount);
    const outsLessThanTotal = outsCmpToTotalAmounts < 0;
    const outsGreaterThanTotal = outsCmpToTotalAmounts > 0;
    const outsEqualToTotal = outsCmpToTotalAmounts === 0;
    // what follows is comparision of the sum of outs amounts
    // vs the total amount actually needed
    // while also building up a list of addresses to send to
    // along with the amounts
    if (outsLessThanTotal) {
        throw errors_1.ERR.BAL.insuff(usingOutsAmount, totalAmount);
    }
    // Now we can put together the list of fund transfers we need to perform
    // excluding the tx fee
    // since that is included in the tx in its own field
    const fundTargets = []; // to build…
    // I. the actual transaction the user is asking to do
    fundTargets.push({
        address: targetAddress,
        amount: feelessTotal,
    });
    // the fee that the hosting provider charges
    // NOTE: The fee has been removed for RCT until a later date
    // fundTransferDescriptions.push({
    //			 address: hostedMoneroAPIClient.HostingServiceFeeDepositAddress(),
    //			 amount: hostingService_chargeAmount
    // })
    // some amount of the total outputs will likely need to be returned to the user as "change":
    if (outsGreaterThanTotal) {
        if (isSweeping) {
            throw errors_1.ERR.SWEEP.TOTAL_NEQ_OUTS;
        }
        // where the change amount is whats left after sending to other addresses + fee
        const changeAmount = usingOutsAmount.subtract(totalAmount);
        logger_1.Log.Amount.change(changeAmount);
        if (isRingCT) {
            // for RCT we don't presently care about dustiness so add entire change amount
            logger_1.Log.Amount.toSelf(changeAmount, senderAddress);
            fundTargets.push({
                address: senderAddress,
                amount: changeAmount,
            });
        }
        else {
            // pre-ringct
            // do not give ourselves change < dust threshold
            const [quotient, remainder] = changeAmount.divRem(xmr_constants_1.config.dustThreshold);
            logger_1.Log.Amount.changeAmountDivRem([quotient, remainder]);
            if (!remainder.isZero()) {
                // miners will add dusty change to fee
                logger_1.Log.Fee.belowDustThreshold(remainder);
            }
            if (!quotient.isZero()) {
                // send non-dusty change to our address
                const usableChange = quotient.multiply(xmr_constants_1.config.dustThreshold);
                logger_1.Log.Amount.toSelf(usableChange, senderAddress);
                fundTargets.push({
                    address: senderAddress,
                    amount: usableChange,
                });
            }
        }
    }
    else if (outsEqualToTotal) {
        // if outputs are equivalent to the total amount
        // this should always fire when sweeping
        // since we want to spend all outputs anyway
        if (isRingCT) {
            // then create random destination to keep 2 outputs always in case of 0 change
            // so we dont create 1 output (outlier)
            const fakeAddress = xmr_crypto_utils_1.create_address(xmr_crypto_utils_1.random_scalar(), nettype)
                .public_addr;
            logger_1.Log.Output.uniformity(fakeAddress);
            fundTargets.push({
                address: fakeAddress,
                amount: biginteger_1.BigInt.ZERO,
            });
        }
    }
    return { fundTargets };
}
exports.validateAndConstructFundTargets = validateAndConstructFundTargets;
// #endregion validateAndConstructFundTargets
//#region constructTx
async function constructTx(params) {
    const { signedTx } = await makeSignedTx(params);
    xmr_str_utils_1.JSONPrettyPrint("constructTx", { signedTx }, "signedTx");
    const { serializedSignedTx, txHash } = getSerializedTxAndHash(signedTx);
    xmr_str_utils_1.JSONPrettyPrint("constructTx", { serializedSignedTx, txHash }, "serializedTxAndHash");
    const { numOfKB } = getTxSize(serializedSignedTx, params.networkFee);
    xmr_str_utils_1.JSONPrettyPrint("constructTx", { numOfKB }, "numOfKb");
    return { numOfKB, txHash, serializedSignedTx };
}
exports.constructTx = constructTx;
async function makeSignedTx(params) {
    try {
        const { senderPublicKeys, senderPrivateKeys, targetAddress, fundTargets, pid, encryptPid, mixOuts, mixin, usingOuts, networkFee, isRingCT, nettype, hwdev, } = params;
        logger_1.Log.Target.fullDisplay(fundTargets);
        const targetViewKey = key_utils_1.getTargetPubViewKey(encryptPid, targetAddress, nettype);
        const splitDestinations = xmr_money_1.decompose_tx_destinations(fundTargets, isRingCT);
        logger_1.Log.Target.displayDecomposed(splitDestinations);
        const signedTx = await xmr_transaction_1.create_transaction(senderPublicKeys, Object.assign({}, senderPrivateKeys, { view: xmr_crypto_utils_2.isRealDevice(hwdev)
                ? (await hwdev.get_secret_keys()).viewKey
                : senderPrivateKeys.view }), splitDestinations, usingOuts, mixOuts, mixin, networkFee, pid, encryptPid, targetViewKey, 0, isRingCT, nettype, hwdev);
        logger_1.Log.Transaction.signed(signedTx);
        return { signedTx };
    }
    catch (e) {
        throw errors_1.ERR.TX.failure(e);
    }
}
function getSerializedTxAndHash(signedTx) {
    // pre rct
    if (signedTx.version === 1) {
        const serializedSignedTx = xmr_transaction_1.serialize_non_rct_tx(signedTx);
        const txHash = xmr_fast_hash_1.cn_fast_hash(serializedSignedTx);
        const ret = {
            serializedSignedTx,
            txHash,
        };
        logger_1.Log.Transaction.serializedAndHash(serializedSignedTx, txHash);
        return ret;
    }
    // rct
    else {
        const { raw, hash } = xmr_transaction_1.serialize_rct_tx_with_hash(signedTx);
        const ret = {
            serializedSignedTx: raw,
            txHash: hash,
        };
        logger_1.Log.Transaction.serializedAndHash(raw, hash);
        return ret;
    }
}
function getTxSize(serializedSignedTx, estMinNetworkFee) {
    // work out per-kb fee for transaction and verify that it's enough
    const txBlobBytes = serializedSignedTx.length / 2;
    let numOfKB = Math.floor(txBlobBytes / 1024);
    if (txBlobBytes % 1024) {
        numOfKB++;
    }
    logger_1.Log.Fee.txKB(txBlobBytes, numOfKB, estMinNetworkFee);
    return { numOfKB };
}
// #endregion constructTx
//# sourceMappingURL=tx_utils.js.map