"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fee_utils_1 = require("./internal_libs/fee_utils");
const mixin_utils_1 = require("./mixin_utils");
const status_update_constants_1 = require("./status_update_constants");
const errors_1 = require("./internal_libs/errors");
const logger_1 = require("./internal_libs/logger");
const parse_target_1 = require("./internal_libs/parse_target");
const pid_utils_1 = require("./internal_libs/pid_utils");
const construct_tx_and_send_1 = require("./internal_libs/construct_tx_and_send");
const biginteger_1 = require("@xmr-core/biginteger");
const xmr_transaction_1 = require("@xmr-core/xmr-transaction");
const xmr_money_1 = require("@xmr-core/xmr-money");
const __1 = require("../");
const output_selection_1 = require("./internal_libs/output_selection");
const xmr_crypto_utils_1 = require("@xmr-core/xmr-crypto-utils");
const xmr_str_utils_1 = require("@xmr-core/xmr-str-utils");
function estimatedTransactionNetworkFee(nonZeroMixin, feePerKB, simplePriority) {
    const numOfInputs = 2; // this might change -- might select inputs
    const numOfOutputs = 1 /*dest*/ + 1 /*change*/ + 0; /*no mymonero fee presently*/
    // TODO: update est tx size for bulletproofs
    // TODO: normalize est tx size fn naming
    const estimatedTxSize = xmr_transaction_1.estimateRctSize(numOfInputs, nonZeroMixin, numOfOutputs);
    const estFee = fee_utils_1.calculateFee(feePerKB, estimatedTxSize, fee_utils_1.multiplyFeePriority(simplePriority));
    return estFee;
}
exports.estimatedTransactionNetworkFee = estimatedTransactionNetworkFee;
async function sendFundsSimple(targetAddress, amount, pid, updateStatus, hwdev) {
    if (!xmr_crypto_utils_1.isRealDevice(hwdev)) {
        throw Error("[sendFundsSimple] This function can only be used with a real hardware device");
    }
    const { spend_public_key, view_public_key, } = await hwdev.get_public_address();
    const address = await xmr_crypto_utils_1.getAddressString(hwdev);
    const senderPrivateKeys = {
        spend: (await hwdev.get_secret_keys()).spendKey,
        view: await hwdev.export_private_view_key(),
    };
    const senderPublicKeys = {
        spend: spend_public_key,
        view: view_public_key,
    };
    return await sendFunds(targetAddress, xmr_crypto_utils_1.NetType.MAINNET, amount, false, address, senderPrivateKeys, senderPublicKeys, pid, mixin_utils_1.fixedMixin(), fee_utils_1.DEFAULT_FEE_PRIORITY, hwdev, updateStatus);
}
exports.sendFundsSimple = sendFundsSimple;
async function sendFundsRawKeys(targetAddress, amount, senderAddress, senderPrivateKeys, senderPublicKeys, updateStatus) {
    return await sendFunds(targetAddress, xmr_crypto_utils_1.NetType.MAINNET, amount, false, senderAddress, senderPrivateKeys, senderPublicKeys, null, mixin_utils_1.fixedMixin(), fee_utils_1.DEFAULT_FEE_PRIORITY, new xmr_crypto_utils_1.DefaultDevice(), updateStatus);
}
exports.sendFundsRawKeys = sendFundsRawKeys;
async function sendFunds(targetAddress, // currency-ready wallet address, but not an OpenAlias address (resolve before calling)
nettype, amountOrZeroWhenSweep, // n value will be ignored for sweep
isSweeping, // send true to sweep - amountorZeroWhenSweep will be ignored
senderAddress, senderPrivateKeys, senderPublicKeys, pidToParse, mixin, simplePriority, hwdev, updateStatus, outputAndAmountSelector = output_selection_1.selectOutputsAndAmountForMixin, api = __1.MyMoneroApi) {
    xmr_str_utils_1.JSONPrettyPrint("sendFunds", {
        targetAddress,
        nettype,
        amountOrZeroWhenSweep,
        isSweeping,
        senderAddress,
        senderPrivateKeys,
        senderPublicKeys,
        pidToParse,
        mixin,
        simplePriority,
    }, "args");
    const isRingCT = true;
    if (mixin < mixin_utils_1.minMixin()) {
        throw errors_1.ERR.RING.INSUFF;
    }
    // parse & normalize the target descriptions by mapping them to Monero addresses & amounts
    const targetAmount = isSweeping ? 0 : amountOrZeroWhenSweep;
    const target = {
        address: targetAddress,
        amount: targetAmount,
    };
    const [singleTarget] = parse_target_1.parseTargets([target], // requires a list of descriptions - but SendFunds was
    // not written with multiple target support as MyMonero does not yet support it
    nettype);
    if (!singleTarget) {
        throw errors_1.ERR.DEST.INVAL;
    }
    const { address, amount } = singleTarget;
    const feelessTotal = new biginteger_1.BigInt(amount);
    logger_1.Log.Amount.beforeFee(feelessTotal, isSweeping);
    if (!isSweeping && feelessTotal.compare(0) <= 0) {
        throw errors_1.ERR.AMT.INSUFF;
    }
    const pidData = pid_utils_1.checkAddressAndPidValidity(address, nettype, pidToParse);
    updateStatus(status_update_constants_1.sendFundStatus.fetchingLatestBalance);
    const { per_kb_fee: feePerKB, unusedOuts } = await api.unspentOutputs(senderAddress, senderPrivateKeys.view, senderPublicKeys.spend, senderPrivateKeys.spend, mixin, hwdev);
    // Transaction will need at least 1KB fee (or 13KB for RingCT)
    const minNetworkTxSizeKb = /*isRingCT ? */ 13; /* : 1*/
    const estMinNetworkFee = fee_utils_1.calculateFeeKb(feePerKB, minNetworkTxSizeKb, fee_utils_1.multiplyFeePriority(simplePriority));
    // construct commonly used parameters
    const senderkeys = {
        senderAddress,
        senderPublicKeys,
        senderPrivateKeys,
    };
    const targetData = {
        targetAddress,
        targetAmount,
    };
    const feeMeta = {
        simplePriority,
        feelessTotal,
        feePerKB,
    };
    const txMeta = {
        isRingCT,
        isSweeping,
        nettype,
    };
    const externApis = {
        updateStatus,
        api,
    };
    // begin the network fee with the smallest fee possible
    let networkFee = estMinNetworkFee;
    // this loop should only execute at most twice
    // 1st execution to generate the inital transaction
    // 2nd execution if the initial transaction's fee is greater than
    // what the predicted tx fee would be
    while (true) {
        // now we're going to try using this minimum fee but the function will be called again
        // if we find after constructing the whole tx that it is larger in kb than
        // the minimum fee we're attempting to send it off with
        const { mixOuts, fundTargets, newFee, usingOuts, } = await construct_tx_and_send_1.getRestOfTxData(Object.assign({}, senderkeys, targetData, { mixin,
            unusedOuts }, feeMeta, { networkFee }, txMeta, externApis), outputAndAmountSelector);
        networkFee = newFee; // reassign network fee to the new fee returned
        const { txFee, txHash, success } = await construct_tx_and_send_1.createTxAndAttemptToSend(Object.assign({}, senderkeys, targetData, { fundTargets }, pidData, { mixin,
            mixOuts,
            usingOuts }, feeMeta, { networkFee }, txMeta, externApis, { hwdev }));
        if (success) {
            const sentAmount = isSweeping
                ? parseFloat(xmr_money_1.formatMoneyFull(feelessTotal))
                : targetAmount;
            return {
                pid: pidData.pid,
                sentAmount,
                targetAddress,
                txFee,
                txHash,
            };
        }
        else {
            // if the function call failed
            // means that we need a higher fee that was returned
            // so reassign network fee to it
            networkFee = txFee;
        }
    }
}
exports.sendFunds = sendFunds;
//# sourceMappingURL=sending_funds.js.map